% VALIDATION_3DFIBREDETECTION
% Written by Monica Jane Emerson, April 2019.
% MIT license

%% Add paths of Insegt, functions and scripts, and prepare workspace
close all, clear all,
addpath('./scripts')
addpath('./functions')

%% Get the measured fibre tracks in the format required by the validation GUI
% USER INPUT: Load the fibre tracks to validate
[filename_tracks,pathname_tracks] = uigetfile('*.mat','Choose the fibre tracks to validate','saved_data/fibreTracks');
load([pathname_tracks,filename_tracks]);

%USER INPUT: Deregister the tracks if necessary
x = questdlg('Have the tracks undergone a transformation (rotation and/or translation) since they were measured from the CT scan?','2D de-registration','Yes','No','No');
if strcmp(x,'Yes')
    [filename,pathname] = uigetfile('*.mat','Load the transformation matrix','saved_data/2Dregistration/');
    t = load([pathname,filename],'tform');  
    %USER INPUT: Offset between registration FoV and analysis FoV
    offset = str2double(inputdlg('Offset between the FoV used for the analysis and the FoV used for the 2D registration','Offset registration-analysis',[1,50],{num2str(100)}));
    aux = [fibres.x(:) + offset*ones(size(fibres.x(:))),fibres.y(:) + offset*ones(size(fibres.x(:))),ones(size(fibres.y(:)))]*t.tform.tdata.Tinv;
    fibres.x = reshape(aux(:,1), size(fibres.x)) - offset*ones(size(fibres.x));
    fibres.y = reshape(aux(:,2), size(fibres.x)) - offset*ones(size(fibres.x));
end

for l = 1:size(fibres.x,2) 
    centers{l} = [fibres.x(:,l),fibres.y(:,l),[1:size(fibres.x,1)]']; 
end;

%% Get the analysed volume of interest (VoI) in the format read by the validaiton GUI
close all
% USER INPUT: Option to open a volume of interest saved as a tif file
x = questdlg('Is the analysed volume of interest saved as one .tif file, with the slices ordered in the direction of tracking?','Load analysed volume','Yes','No','Yes');

if strcmp(x,'No')
    indicate_dataFolder;
    [V,depth] = loadVolumeRoI(path_volumeFolder);
    % USER INPUT: Tracking direction
    top = ['Top to bottom (from slice ',depth{1},')'];
    bottom = ['Bottom to top (from slice ',depth{2},')'];
    direct = questdlg('In which direction did you track the coordinates?','Tracking direction',top, bottom, bottom);
    % USER INPUT: Give a name to the .tif Volume of Interest
    if strcmp(direct,bottom)
        direction = [depth{2},'to',depth{1},'step',depth{3}];
    else
        direction = [depth{1},'to',depth{2},'step',depth{3}];
    end
    name_tif = inputdlg('Name for the .tif file of your volume of interest (VoI): ','Name your VOI',[1,55],{['V_?_',direction,'.tif']});
    filename_voi= name_tif{1};
    pathname_voi = 'saved_data/volumeOfInterest/';
    if strcmp(direct,bottom) 
        imwrite(V(:,:,end),[pathname_voi,filename_voi]);
        for l= 2:size(V,3)
            imwrite(V(:,:,end-l+1),[pathname_voi,filename_voi], 'WriteMode','append');
        end 
    else
        imwrite(V(:,:,1),[pathname_voi,filename_voi]);
        for l= 2:size(V,3)
            imwrite(V(:,:,l),[pathname_voi,filename_voi], 'WriteMode','append');
        end 
    end
else
    [filename_voi,pathname_voi] = uigetfile('*.tif','Choose volume of interest','saved_data/volumeOfInterest/VOI_v');
end

%% Validate all fibres
track_slicer([pathname_voi,filename_voi],centers)
